"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const connection_1 = require("../connection");
const AssociationsParser_1 = require("./AssociationsParser");
const utils_1 = require("./utils");
class Dialect {
    /**
     * Build tables metadata for the specific dialect and schema
     * @param {IConfig} config
     * @returns {Promise<ITableMetadata[]>}
     */
    buildTablesMetadata(config) {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        return __awaiter(this, void 0, void 0, function* () {
            let connection;
            const tablesMetadata = {};
            try {
                // Set schema for Postgres to 'public' if not provided
                if (config.connection.dialect === 'postgres' && !((_a = config.metadata) === null || _a === void 0 ? void 0 : _a.schema)) {
                    config.metadata = Object.assign(Object.assign({}, config.metadata), { schema: 'public' });
                }
                connection = connection_1.createConnection(config.connection);
                yield connection.authenticate();
                let tables = yield this.fetchTables(connection, config);
                // Apply filters
                tables = tables
                    .filter(({ name }) => {
                    var _a, _b;
                    if ((_b = (_a = config.metadata) === null || _a === void 0 ? void 0 : _a.tables) === null || _b === void 0 ? void 0 : _b.length) {
                        return config.metadata.tables.includes(name.toLowerCase());
                    }
                    else {
                        return true;
                    }
                }).filter(({ name }) => {
                    var _a, _b;
                    if ((_b = (_a = config.metadata) === null || _a === void 0 ? void 0 : _a.skipTables) === null || _b === void 0 ? void 0 : _b.length) {
                        return !(config.metadata.skipTables.includes(name.toLowerCase()));
                    }
                    else {
                        return true;
                    }
                });
                for (const { name: tableName, comment: tableComment } of tables) {
                    const columnsMetadata = yield this.fetchColumnsMetadata(connection, config, tableName);
                    // Fetch indices metadata if required
                    if ((_b = config.metadata) === null || _b === void 0 ? void 0 : _b.indices) {
                        for (const column of columnsMetadata) {
                            column.indices = yield this.fetchColumnIndexMetadata(connection, config, tableName, column.name);
                        }
                    }
                    const tableMetadata = Object.assign(Object.assign({ originName: tableName, name: tableName }, ((_c = config.metadata) === null || _c === void 0 ? void 0 : _c.schema) && { schema: config.metadata.schema }), { timestamps: (_e = (_d = config.metadata) === null || _d === void 0 ? void 0 : _d.timestamps, (_e !== null && _e !== void 0 ? _e : false)), columns: {}, comment: (tableComment !== null && tableComment !== void 0 ? tableComment : undefined) });
                    for (const columnMetadata of columnsMetadata) {
                        tableMetadata.columns[columnMetadata.name] = columnMetadata;
                    }
                    tablesMetadata[tableMetadata.originName] = tableMetadata;
                }
            }
            catch (err) {
                console.error(err);
                process.exit(1);
            }
            finally {
                connection && (yield connection.close());
            }
            // Apply associations if required
            if ((_f = config.metadata) === null || _f === void 0 ? void 0 : _f.associationsFile) {
                const parsedAssociations = AssociationsParser_1.AssociationsParser.parse((_g = config.metadata) === null || _g === void 0 ? void 0 : _g.associationsFile);
                for (const [tableName, association] of Object.entries(parsedAssociations)) {
                    if (!tablesMetadata[tableName]) {
                        console.warn('[WARNING]', `Associated table ${tableName} not found among (${Object.keys(tablesMetadata).join(', ')})`);
                        continue;
                    }
                    // Attach associations to table
                    tablesMetadata[tableName].associations = association.associations;
                    const { columns } = tablesMetadata[tableName];
                    // Override foreign keys
                    for (const { name: columnName, targetModel } of association.foreignKeys) {
                        if (!columns[columnName]) {
                            console.warn('[WARNING]', `Foreign key column ${columnName} not found among (${Object.keys(columns).join(', ')})`);
                            continue;
                        }
                        columns[columnName].foreignKey = {
                            name: columnName,
                            targetModel: targetModel
                        };
                    }
                }
            }
            // Apply transformations if required
            if ((_h = config.metadata) === null || _h === void 0 ? void 0 : _h.case) {
                for (const [tableName, tableMetadata] of Object.entries(tablesMetadata)) {
                    tablesMetadata[tableName] = utils_1.caseTransformer(tableMetadata, config.metadata.case);
                }
            }
            return tablesMetadata;
        });
    }
}
exports.Dialect = Dialect;
/**
 * Accepted dialects
 */
Dialect.dialects = new Set([
    'postgres',
    'mysql',
    'mariadb',
    'sqlite',
    'mssql',
]);
