"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const change_case_1 = require("change-case");
exports.toUpperCase = (s) => s.toUpperCase();
exports.toLowerCase = (s) => s.toLowerCase();
/**
 * Return transformer for the provided case
 * @param  {TransformCase} transformCase
 * @returns {function(s: string): string}
 */
exports.getTransformer = (transformCase) => {
    let transformer;
    switch (transformCase) {
        case "CAMEL":
            transformer = change_case_1.camelCase;
            break;
        case "UPPER":
            transformer = exports.toUpperCase;
            break;
        case "LOWER":
            transformer = exports.toLowerCase;
            break;
        case "PASCAL":
            transformer = change_case_1.pascalCase;
            break;
        case "UNDERSCORE":
            transformer = change_case_1.snakeCase;
            break;
        case "CONST":
            transformer = change_case_1.constantCase;
            break;
        default:
            transformer = (s) => s;
    }
    return transformer;
};
/**
 * Transform ITableMetadata object using the provided case
 * @param {ITableMetadata} tableMetadata
 * @param {TransformCase} transformCase
 * @returns {ITableMetadata}
 */
exports.caseTransformer = (tableMetadata, transformCase) => {
    const transformer = exports.getTransformer(transformCase);
    const transformed = Object.assign(Object.assign({ originName: tableMetadata.originName, name: transformer(tableMetadata.originName), timestamps: tableMetadata.timestamps, columns: {} }, tableMetadata.associations && {
        associations: tableMetadata.associations.map(a => {
            a.targetModel = transformer(a.targetModel);
            if (a.joinModel) {
                a.joinModel = transformer(a.joinModel);
            }
            if (a.sourceKey) {
                a.sourceKey = transformer(a.sourceKey);
            }
            return a;
        })
    }), { comment: tableMetadata.comment });
    for (const [columnName, columnMetadata] of Object.entries(tableMetadata.columns)) {
        if (columnMetadata.foreignKey) {
            const { name, targetModel } = columnMetadata.foreignKey;
            columnMetadata.foreignKey = {
                name: transformer(name),
                targetModel: transformer(targetModel),
            };
        }
        transformed.columns[columnName] = Object.assign({}, columnMetadata, { name: transformer(columnMetadata.originName) });
    }
    return transformed;
};
/**
 * Unknown mapping warning
 * @param {string} dataType
 * @returns {string}
 */
exports.warnUnknownMappingForDataType = (dataType) => {
    console.warn(`[Warning]`, `Unknown data type mapping for type '${dataType}'. 
        You should define the data type manually.     
    `);
};
/**
 * Generates precision signature
 * @param {Array<string|number>} args
 * @returns {string} (80) or (10,4) or ...
 */
exports.generatePrecisionSignature = (...args) => {
    const tokens = args.filter(arg => !!arg);
    return tokens.length ? `(${tokens.join(',')})` : '';
};
